import Foundation
import Combine

enum APIError: Error {
    case invalidURL
    case noData
    case decodingError
    case networkError(String)
    case unauthorized
}

class APIClient {
    static let shared = APIClient()
    
    #if DEBUG
    private let baseURL = "http://localhost:8000/api/v1"
    #else
    private let baseURL = "https://api.card.syui.ai/api/v1"
    #endif
    
    private var cancellables = Set<AnyCancellable>()
    
    private init() {}
    
    private var authToken: String? {
        get { UserDefaults.standard.string(forKey: "authToken") }
        set { UserDefaults.standard.set(newValue, forKey: "authToken") }
    }
    
    private func request<T: Decodable>(_ endpoint: String,
                                       method: String = "GET",
                                       body: Data? = nil,
                                       authenticated: Bool = true) -> AnyPublisher<T, APIError> {
        guard let url = URL(string: "\(baseURL)\(endpoint)") else {
            return Fail(error: APIError.invalidURL).eraseToAnyPublisher()
        }
        
        var request = URLRequest(url: url)
        request.httpMethod = method
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        
        if authenticated, let token = authToken {
            request.setValue("Bearer \(token)", forHTTPHeaderField: "Authorization")
        }
        
        if let body = body {
            request.httpBody = body
        }
        
        return URLSession.shared.dataTaskPublisher(for: request)
            .tryMap { data, response in
                guard let httpResponse = response as? HTTPURLResponse else {
                    throw APIError.networkError("Invalid response")
                }
                
                if httpResponse.statusCode == 401 {
                    throw APIError.unauthorized
                }
                
                if !(200...299).contains(httpResponse.statusCode) {
                    throw APIError.networkError("Status code: \(httpResponse.statusCode)")
                }
                
                return data
            }
            .decode(type: T.self, decoder: JSONDecoder())
            .mapError { error in
                if error is DecodingError {
                    return APIError.decodingError
                } else if let apiError = error as? APIError {
                    return apiError
                } else {
                    return APIError.networkError(error.localizedDescription)
                }
            }
            .eraseToAnyPublisher()
    }
    
    // MARK: - Auth
    
    func login(identifier: String, password: String) -> AnyPublisher<LoginResponse, APIError> {
        let loginRequest = LoginRequest(identifier: identifier, password: password)
        guard let body = try? JSONEncoder().encode(loginRequest) else {
            return Fail(error: APIError.decodingError).eraseToAnyPublisher()
        }
        
        return request("/auth/login", method: "POST", body: body, authenticated: false)
            .handleEvents(receiveOutput: { [weak self] (response: LoginResponse) in
                self?.authToken = response.accessToken
            })
            .eraseToAnyPublisher()
    }
    
    func logout() -> AnyPublisher<Void, APIError> {
        request("/auth/logout", method: "POST")
            .map { (_: [String: String]) in () }
            .handleEvents(receiveCompletion: { [weak self] _ in
                self?.authToken = nil
            })
            .eraseToAnyPublisher()
    }
    
    func verify() -> AnyPublisher<User, APIError> {
        request("/auth/verify")
    }
    
    // MARK: - Cards
    
    func drawCard(userDid: String, isPaid: Bool = false) -> AnyPublisher<CardDrawResult, APIError> {
        let body = try? JSONEncoder().encode([
            "user_did": userDid,
            "is_paid": isPaid
        ])
        
        return request("/cards/draw", method: "POST", body: body)
    }
    
    func getUserCards(userDid: String) -> AnyPublisher<[Card], APIError> {
        request("/cards/user/\(userDid)")
    }
    
    func getUniqueCards() -> AnyPublisher<[[String: Any]], APIError> {
        request("/cards/unique")
    }
}