import React, { useState, useEffect } from 'react';
import { motion, AnimatePresence } from 'framer-motion';
import { Card } from './Card';
import { Card as CardType } from '../types/card';
import '../styles/GachaAnimation.css';

interface GachaAnimationProps {
  card: CardType;
  animationType: string;
  onComplete: () => void;
}

export const GachaAnimation: React.FC<GachaAnimationProps> = ({
  card,
  animationType,
  onComplete
}) => {
  const [phase, setPhase] = useState<'opening' | 'revealing' | 'complete'>('opening');

  useEffect(() => {
    const timer1 = setTimeout(() => setPhase('revealing'), 1500);
    const timer2 = setTimeout(() => {
      setPhase('complete');
      onComplete();
    }, 3000);

    return () => {
      clearTimeout(timer1);
      clearTimeout(timer2);
    };
  }, [onComplete]);

  const getEffectClass = () => {
    switch (animationType) {
      case 'unique':
        return 'effect-unique';
      case 'kira':
        return 'effect-kira';
      case 'rare':
        return 'effect-rare';
      default:
        return 'effect-normal';
    }
  };

  return (
    <div className={`gacha-container ${getEffectClass()}`}>
      <AnimatePresence mode="wait">
        {phase === 'opening' && (
          <motion.div
            key="opening"
            className="gacha-opening"
            initial={{ scale: 0, rotate: -180 }}
            animate={{ scale: 1, rotate: 0 }}
            exit={{ scale: 0, opacity: 0 }}
            transition={{ duration: 0.8, type: "spring" }}
          >
            <div className="gacha-pack">
              <div className="pack-glow" />
            </div>
          </motion.div>
        )}

        {phase === 'revealing' && (
          <motion.div
            key="revealing"
            initial={{ scale: 0 }}
            animate={{ scale: 1 }}
            transition={{ duration: 0.5, type: "spring" }}
          >
            <Card card={card} isRevealing={true} />
          </motion.div>
        )}
      </AnimatePresence>

      {animationType === 'unique' && (
        <div className="unique-effect">
          <div className="unique-particles" />
          <div className="unique-burst" />
        </div>
      )}
    </div>
  );
};